﻿; DocMaker
; by MAC 09/2019
; Version 1.01

; General tags for documentation layouting
; ----------------------------------------
; ;@title The title of the documentation
; ;@author The author of the documentation
; ;@version The version of the documentation
; ;@chapter The title of the current chapter
; ;@ Description (can be multilined) All descriptions that are not in a procedure, enumeration block or structure block will be considered as chapter descriptions

; Tags for Procedures:
; --------------------
; ;@ Description (can be multilined)
; ;@author The name of the author
; ;@version The version
; ;@param Name Description (A parameter name and a description (\n for line feed))
; ;@event Name
; ;@attribute Name Rights Description (An attribute this procedure processes (\n for line feed))
; ;@return Description of the return value
; ;@link Type Link
; ;@code Some code here
; ;@image An image file

; Tags for Enumerations:
; ----------------------
; ;@name The name of a enumeration list
; ;@ Description (can be multilined)
; ; A commentary behind a constant in a enumeration block will be saved as the description (\n for line feed)

; Tags for Structures:
; --------------------
; ;@ Description (can be multilined)

;- Structures

;--- Str_DocMaker_Link
Structure Str_DocMaker_Link
  Type.s ; "Enum"=Enumeration, "Proc"=Procedure, "Ext"=External
  Link.s
EndStructure

;--- Str_DocMaker_Attribute
Structure Str_DocMaker_Attribute
  Name.s
  Rights.s
  Description.s
EndStructure

;--- Str_DocMaker_Constant
Structure Str_DocMaker_Constant
  Name.s
  Description.s
EndStructure

;--- Str_DocMaker_Enumeration
Structure Str_DocMaker_Enumeration
  Name.s
  Description.s
  Chapter.s
  List Constant.Str_DocMaker_Constant()
EndStructure

;--- Str_DocMaker_Parameter
Structure Str_DocMaker_Parameter
  Name.s
  Description.s
EndStructure

;--- Str_DocMaker_Procedure
Structure Str_DocMaker_Procedure
  Declaration.s
  Description.s
  Author.s
  Version.s
  ReturnValue.s
  Chapter.s
  Code.s
  List Image.s()
  List Link.Str_DocMaker_Link()
  List Param.Str_DocMaker_Parameter()
  List Attribute.Str_DocMaker_Attribute()
  List Event.Str_DocMaker_Parameter()
  List Flag.Str_DocMaker_Parameter()
EndStructure

;--- Str_DocMaker_Include
Structure Str_DocMaker_Include
  File.s
  LastImport.i
  Title.s
  Author.s
  Version.s
  Description.s
  List Procedures.Str_DocMaker_Procedure()
  List Enumerations.Str_DocMaker_Enumeration()
  List Chapters.Str_DocMaker_Parameter()
  List Structures.Str_DocMaker_Enumeration()
EndStructure

;--- Str_DocMaker_Project
Structure Str_DocMaker_Project
  List Include.Str_DocMaker_Include()

  HTML_BGColor.i
  HTML_TextColor.i
  HTML_LinkColor.i
  HTML_LinkColor_Visited.i
  HTML_LinkColor_Active.i

  HTML_Parameter_BGColor.i
  HTML_Parameter_TextColor.i

  HTML_Attribute_BGColor.i
  HTML_Attribute_TextColor.i

  HTML_Enumeration_BGColor.i
  HTML_Enumeration_TextColor.i
  
  Option_WriteEnumerations.b
  Option_WriteProcedures.b
  Option_WriteStructures.b
  Option_WriteAttributes.b
  Option_OneFile.b
  
  Option_WriteTOC.b
  
  TOCColumns.b
EndStructure


;- Procedures

Procedure ScanInclude(*Include.Str_DocMaker_Include, Filename.s)
  InsideProcedure=0
  InsideEnumeration=0
  InsideStructure=0
  CurrentChapter.s=""
  
  file=ReadFile(#PB_Any,Filename,#PB_File_SharedRead|#PB_File_SharedWrite)
  If file
    ClearList(*Include\Procedures())
    While Not Eof(file)
      Line.s=ReadString(file)

      ; Enumeration block processing
      pos=FindString(LCase(Line),"enumeration") ; Found enumeration block
      If pos
        If FindString(LCase(Line),"endenumeration")=0
          Scan=1
          For t=1 To pos
            If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
              Scan=0
              Break
            EndIf
          Next
          If Scan
            InsideEnumeration=1
            AddElement(*Include\Enumerations())
            *Include\Enumerations()\Chapter=CurrentChapter
          EndIf
        EndIf
      EndIf
      
      If InsideEnumeration
        pos=FindString(LCase(Line),"#")
        If pos
          Scan=1
          For t=1 To pos
            If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
              Scan=0
              Break
            EndIf
          Next
          If Scan
            Constant.s=""
            DescPos=0
            For t=pos To Len(Line)
              If Mid(Line,t,1)=";"
                DescPos=t
                Break
              EndIf
              Constant+Mid(Line,t,1)
            Next
            AddElement(*Include\Enumerations()\Constant())
            *Include\Enumerations()\Constant()\Name=Constant
            If DescPos<>0
              *Include\Enumerations()\Constant()\Description=Mid(Line,DescPos+1)
            EndIf
          EndIf
        EndIf

        pos=FindString(Line,";@")
        If pos
          If Mid(LCase(Line),pos+2,4)="name"
            *Include\Enumerations()\Name+Right(Line,Len(Line)-pos-2-4)
          ElseIf Mid(LCase(Line),pos+2,1)=" "
            *Include\Enumerations()\Description+Right(Line,Len(Line)-pos-2)+Chr(10)
          EndIf
        EndIf
      EndIf
      
      pos=FindString(LCase(Line),"endenumeration") ; Found end of enumeration block
      If pos
        Scan=1
        For t=1 To pos
          If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
            Scan=0
            Break
          EndIf
        Next
        If Scan
          InsideEnumeration=0
        EndIf
      EndIf
      
      ; Structure block processing
      pos=FindString(LCase(Line),"structure ") ; Found enumeration block
      If pos=1 Or (pos>1 And Mid(Line,pos-1,1)=" ")
        If FindString(LCase(Line),"endstructure")=0
          Scan=1
          For t=1 To pos
            If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
              Scan=0
              Break
            EndIf
          Next
          If Scan
            InsideStructure=1
            AddElement(*Include\Structures())
            *Include\Structures()\Chapter=CurrentChapter
            *Include\Structures()\Name=Trim(RemoveString(Line,"structure",#PB_String_NoCase,pos-1))
          EndIf
        EndIf
      EndIf
      
      If InsideStructure
        pos=FindString(Line,";@")
        If pos
          If Mid(LCase(Line),pos+2,1)=" "
            *Include\Structures()\Description+Right(Line,Len(Line)-pos-2)+Chr(10)
          EndIf
        EndIf
      EndIf
      
      pos=FindString(LCase(Line),"endstructure") ; Found end of structure block
      If pos
        Scan=1
        For t=1 To pos
          If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
            Scan=0
            Break
          EndIf
        Next
        If Scan
          InsideStructure=0
        EndIf
      EndIf
      
      ; Procedure processing
      pos=FindString(LCase(Line),"procedure") ; Found declaration line
      If pos
        If FindString(LCase(Line),"endprocedure")=0 And FindString(LCase(Line),"procedurereturn")=0
          Scan=1
          For t=1 To pos
            If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
              Scan=0
              Break
            EndIf
          Next
          If Scan
            InsideProcedure=1
            AddElement(*Include\Procedures())
            *Include\Procedures()\Chapter=CurrentChapter
            *Include\Procedures()\Declaration=Trim(RemoveString(Line,"procedure",#PB_String_NoCase,pos-1))
            If Left(*Include\Procedures()\Declaration,1)="." ; Delete procedure return type
              *Include\Procedures()\Declaration=Trim(Right(*Include\Procedures()\Declaration,Len(*Include\Procedures()\Declaration)-2))
            EndIf
          EndIf
        EndIf
      EndIf

      If InsideProcedure
        pos=FindString(LCase(Line),";@") ; Found doc commentary
        If pos
          If Mid(LCase(Line),pos+2,6)="author"
            *Include\Procedures()\Author=Right(Line,Len(Line)-pos-2-6)
            
          ElseIf Mid(LCase(Line),pos+2,7)="version"
            *Include\Procedures()\Version=Right(Line,Len(Line)-pos-2-7)
            
          ElseIf Mid(LCase(Line),pos+2,5)="param"
            AddElement(*Include\Procedures()\Param())
            Param.s=Right(Line,Len(Line)-pos-2-5)
            posspace=FindString(Param," ")
            If posspace>0
              *Include\Procedures()\Param()\Name=Left(Param,posspace)
              *Include\Procedures()\Param()\Description=ReplaceString(Mid(Param,posspace+1),"\n",Chr(10))
            Else
              *Include\Procedures()\Param()\Name=Param
            EndIf
            
          ElseIf Mid(LCase(Line),pos+2,6)="return"
            *Include\Procedures()\ReturnValue=Right(Line,Len(Line)-pos-2-6)
            
          ElseIf Mid(LCase(Line),pos+2,4)="link"
            AddElement(*Include\Procedures()\Link())
            Link.s=Right(Line,Len(Line)-pos-2-4)
            posspace=FindString(Link," ")
            *Include\Procedures()\Link()\Type=LCase(Left(Link,posspace))
            *Include\Procedures()\Link()\Link=Mid(Link,posspace+1)
            
          ElseIf Mid(LCase(Line),pos+2,4)="code"
            *Include\Procedures()\Code+Right(Line,Len(Line)-pos-2-4)+Chr(10)

          ElseIf Mid(LCase(Line),pos+2,5)="image"
            AddElement(*Include\Procedures()\Image())
            *Include\Procedures()\Image()=Right(Line,Len(Line)-pos-2-5)

          ElseIf Mid(LCase(Line),pos+2,9)="attribute"
            AddElement(*Include\Procedures()\Attribute())
            Param.s=Right(Line,Len(Line)-pos-2-9)
            posspace=FindString(Param," ")
            *Include\Procedures()\Attribute()\Name=Left(Param,posspace)
            posspace2=FindString(Param," ",posspace+1)
            If posspace2>0
              *Include\Procedures()\Attribute()\Rights=Mid(Param,posspace+1,posspace2-posspace)
              *Include\Procedures()\Attribute()\Description=ReplaceString(Mid(Param,posspace2+1),"\n",Chr(10))
            Else
              *Include\Procedures()\Attribute()\Rights=Mid(Param,posspace+1)
            EndIf

          ElseIf Mid(LCase(Line),pos+2,5)="event"
            AddElement(*Include\Procedures()\Event())
            *Include\Procedures()\Event()\Name=Right(Line,Len(Line)-pos-2-5)

          ElseIf Mid(LCase(Line),pos+2,1)=" " ; Description
            *Include\Procedures()\Description+Right(Line,Len(Line)-pos-2)+Chr(10)
            
          EndIf
        EndIf
      EndIf

      pos=FindString(LCase(Line),"endprocedure")
      If pos
        Scan=1
        For t=1 To pos
          If Mid(Line,t,1)=";" Or Mid(Line,t,1)=Chr(34)
            Scan=0
            Break
          EndIf
        Next
        If Scan
          InsideProcedure=0
        EndIf
      EndIf
      
      ; General processing
      If InsideEnumeration=0 And InsideProcedure=0 And InsideStructure=0
        pos=FindString(LCase(Line),";@") ; Found doc commentary
        If pos
          If Mid(LCase(Line),pos+2,7)="chapter"
            CurrentChapter=Right(Line,Len(Line)-pos-2-7)
            ChapterFound=0
            ForEach *Include\Chapters()
              If *Include\Chapters()\Name=CurrentChapter
                ChapterFound=1
                Break
              EndIf
            Next
            If ChapterFound=0
              AddElement(*Include\Chapters())
              *Include\Chapters()\Name=CurrentChapter
            EndIf

          ElseIf Mid(LCase(Line),pos+2,5)="title"
            *Include\Title=Right(Line,Len(Line)-pos-2-5)

          ElseIf Mid(LCase(Line),pos+2,6)="author"
            *Include\Author=Right(Line,Len(Line)-pos-2-6)
            
          ElseIf Mid(LCase(Line),pos+2,7)="version"
            *Include\Version=Right(Line,Len(Line)-pos-2-7)
            
          ElseIf Mid(LCase(Line),pos+2,1)=" " ; Description
            If CurrentChapter<>""
              *Include\Chapters()\Description+Right(Line,Len(Line)-pos-2)+Chr(10)
            Else
              *Include\Description+Right(Line,Len(Line)-pos-2)+Chr(10)
            EndIf
            
          EndIf
        EndIf            
      EndIf
      
    Wend
    CloseFile(file)
    *Include\LastImport=GetFileDate(Filename,#PB_Date_Modified)
  Else
    MessageRequester("Import Include","Error: The file '"+Filename+"' couldn't be opened.")
  EndIf
EndProcedure

CompilerIf #PB_Compiler_IsMainFile

  Global Project.Str_DocMaker_Project
  XIncludeFile "..\..\sqUId\sqUId.pbi"
  XIncludeFile "DocMaker_UI.pb"
  
  Procedure ClearIncludes()
    ForEach Project\Include()
      ClearList(Project\Include()\Chapters())
      ClearList(Project\Include()\Enumerations())
      ClearList(Project\Include()\Procedures())
    Next
  EndProcedure

  Procedure AddInclude()
    Source.s=OpenFileRequester("Open source file","","PureBasic files|*.pb;*.pbi",0)
    If Source<>""
      AddElement(Project\Include())
      Project\Include()\File=Source
      UI_AddGadgetItem(#Table_Includes,-1,GetFilePart(Source))
      UI_SetGadgetItemData(#Table_Includes,-1,@Project\Include())
    EndIf
  EndProcedure
  
  Procedure DeleteInclude()
    If UI_GetGadgetState(#Table_Includes)<>-1
      ChangeCurrentElement(Project\Include(),UI_GetGadgetItemData(#Table_Includes,UI_GetGadgetState(#Table_Includes)))
      DeleteElement(Project\Include())
      UI_RemoveGadgetItem(#Table_Includes,UI_GetGadgetState(#Table_Includes))
    EndIf
  EndProcedure
  
  Procedure UpdateWindow()
    UI_SetGadgetState(#Checkbox_WriteTOC,Project\Option_WriteTOC)
    UI_SetGadgetState(#Checkbox_WriteProcedures,Project\Option_WriteProcedures)
    UI_SetGadgetState(#Checkbox_WriteEnumerations,Project\Option_WriteEnumerations)
    UI_SetGadgetState(#Checkbox_WriteStructures,Project\Option_WriteStructures)
    
    If (Red(Project\HTML_BGColor)+Green(Project\HTML_BGColor)+Blue(Project\HTML_BGColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_BGColor,#UI_ThemeStyle_All,Project\HTML_BGColor, #PB_Ignore, TextCol)

    If (Red(Project\HTML_TextColor)+Green(Project\HTML_TextColor)+Blue(Project\HTML_TextColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_TextColor, #UI_ThemeStyle_All, Project\HTML_TextColor, #PB_Ignore, TextCol)

    If (Red(Project\HTML_LinkColor)+Green(Project\HTML_LinkColor)+Blue(Project\HTML_LinkColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_LinkColor, #UI_ThemeStyle_All, Project\HTML_LinkColor, #PB_Ignore, TextCol)
    
    If (Red(Project\HTML_LinkColor_Visited)+Green(Project\HTML_LinkColor_Visited)+Blue(Project\HTML_LinkColor_Visited))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_LinkVisitedColor, #UI_ThemeStyle_All, Project\HTML_LinkColor_Visited, #PB_Ignore, TextCol)

    If (Red(Project\HTML_LinkColor_Active)+Green(Project\HTML_LinkColor_Active)+Blue(Project\HTML_LinkColor_Active))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_LinkActiveColor, #UI_ThemeStyle_All, Project\HTML_LinkColor_Active, #PB_Ignore, TextCol)
    
    If (Red(Project\HTML_Parameter_BGColor)+Green(Project\HTML_Parameter_BGColor)+Blue(Project\HTML_Parameter_BGColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_ParameterBGColor, #UI_ThemeStyle_All, Project\HTML_Parameter_BGColor, #PB_Ignore, TextCol)
    
    If (Red(Project\HTML_Parameter_TextColor)+Green(Project\HTML_Parameter_TextColor)+Blue(Project\HTML_Parameter_TextColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_ParameterTextColor, #UI_ThemeStyle_All, Project\HTML_Parameter_TextColor, #PB_Ignore, TextCol)

    If (Red(Project\HTML_Enumeration_BGColor)+Green(Project\HTML_Enumeration_BGColor)+Blue(Project\HTML_Enumeration_BGColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_EnumerationBGColor, #UI_ThemeStyle_All, Project\HTML_Enumeration_BGColor, #PB_Ignore, TextCol)

    If (Red(Project\HTML_Enumeration_TextColor)+Green(Project\HTML_Enumeration_TextColor)+Blue(Project\HTML_Enumeration_TextColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_EnumerationTextColor, #UI_ThemeStyle_All, Project\HTML_Enumeration_TextColor, #PB_Ignore, TextCol)
    
    If (Red(Project\HTML_Attribute_BGColor)+Green(Project\HTML_Attribute_BGColor)+Blue(Project\HTML_Attribute_BGColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_Attribute_BGColor, #UI_ThemeStyle_All, Project\HTML_Attribute_BGColor, #PB_Ignore, TextCol)

    If (Red(Project\HTML_Attribute_TextColor)+Green(Project\HTML_Attribute_TextColor)+Blue(Project\HTML_Attribute_TextColor))/3 > 128
      TextCol=RGBA(0,0,0,255)
    Else
      TextCol=RGBA(255,255,255,255)
    EndIf
    UI_SetGadgetColors(#Button_Attribute_TextColor, #UI_ThemeStyle_All, Project\HTML_Attribute_TextColor, #PB_Ignore, TextCol)
    
    If UI_GetGadgetState(#Table_Includes)=-1
      UI_DisableGadget(#Button_DelInclude,1)
      UI_DisableGadget(#Button_Up,1)
      UI_DisableGadget(#Button_Down,1)
    Else
      UI_DisableGadget(#Button_DelInclude,0)
      If UI_GetGadgetState(#Table_Includes)=0
        UI_DisableGadget(#Button_Up,1)
      Else
        UI_DisableGadget(#Button_Up,0)
      EndIf  
      UI_DisableGadget(#Button_Down,0)
      If UI_GetGadgetState(#Table_Includes)=UI_CountGadgetItems(#Table_Includes)-1
        UI_DisableGadget(#Button_Down,1)
      Else
        UI_DisableGadget(#Button_Down,0)
      EndIf  
    EndIf

  EndProcedure
  
  Procedure.s ColorToHex(Color)
    ProcedureReturn RSet(Hex(Red(Color),#PB_Byte),2,"0") + RSet(Hex(Green(Color),#PB_Byte),2,"0") + RSet(Hex(Blue(Color),#PB_Byte),2,"0")
  EndProcedure

  Procedure ExportHTML(Filename.s)
    ; Compile all includes into one
    FirstElement(Project\Include())
    *Include.Str_DocMaker_Include=@Project\Include()
    While NextElement(Project\Include())
      MergeLists(Project\Include()\Enumerations(),*Include\Enumerations())
      MergeLists(Project\Include()\Procedures(),*Include\Procedures())
      MergeLists(Project\Include()\Structures(),*Include\Structures())
      ForEach Project\Include()\Chapters()
        Found=0
        ForEach *Include\Chapters()
          If *Include\Chapters()\Name=Project\Include()\Chapters()\Name
            Found=1
            Break
          EndIf
        Next
        If Found=0
          AddElement(*Include\Chapters())
          *Include\Chapters()\Name=Project\Include()\Chapters()\Name
          *Include\Chapters()\Description=Project\Include()\Chapters()\Description
        EndIf
      Next
    Wend
    
    ; Create HTML file
    file=CreateFile(#PB_Any,Filename)
    If file
      Found=0
      ForEach *Include\Chapters()
        If *Include\Chapters()\Name=""
          Found=1
          Break
        EndIf
      Next
      If Found=0
        FirstElement(*Include\Chapters())
        InsertElement(*Include\Chapters())
        *Include\Chapters()\Description=*Include\Description
      EndIf
  
      ; Write header
      WriteStringN(file,"<html>")
      WriteStringN(file,"<head>")
      WriteStringN(file,"  <style type="+Chr(34)+"text/css"+Chr(34)+">")
      WriteStringN(file,"    body { background-color:#"+ColorToHex(Project\HTML_BGColor)+"; color:#"+ColorToHex(Project\HTML_TextColor)+"; margin-left:50px; margin-right:50px; margin-top:100px; margin-bottom:100px }")
      WriteStringN(file,"    a:link { color:#"+ColorToHex(Project\HTML_LinkColor)+"; }")
      WriteStringN(file,"    a:visited { color:#"+ColorToHex(Project\HTML_LinkColor_Visited)+"; }")
      WriteStringN(file,"    a:active { color:#"+ColorToHex(Project\HTML_LinkColor_Active)+"; }")
      WriteStringN(file,"  </style>")
      WriteStringN(file,"</head>")
      WriteStringN(file,"")
      
      ; Write body
      WriteStringN(file,"<body>")
        
        ; Write title
        WriteStringN(file,"<center>")
        If *Include\Title<>""
          WriteStringN(file,"<h1>"+*Include\Title+"</h1>")
        Else
          WriteStringN(file,"<h1>Documentation for file "+GetFilePart(*Include\File)+"</h1>")
        EndIf
        If *Include\Author<>""
          WriteStringN(file,"by "+*Include\Author+"<br>")
        EndIf
        If *Include\Version<>""
          WriteStringN(file,"Version "+*Include\Version+"<br>")
        EndIf
        WriteStringN(file,"</center>")
        
        ; Table of content
        If Project\Option_WriteTOC
          WriteStringN(file,"<center><h2>Content</h2></center>")
          ForEach *Include\Chapters()
            WriteStringN(file,"<h3>"+*Include\Chapters()\Name+"</h3>")
            
            If ListSize(*Include\Enumerations())>0 And Project\Option_WriteEnumerations
              Found=0
              ForEach *Include\Enumerations()
                If *Include\Enumerations()\Chapter=*Include\Chapters()\Name
                  Found=1
                  Break
                EndIf
              Next
              If Found
                WriteStringN(file,"<h4>Enumerations</h4>")
                WriteStringN(file,"<table border=1 width="+Chr(34)+"100%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" cellspacing=0 rules="+Chr(34)+"none"+Chr(34)+">")
                WriteStringN(file,"<colgroup>")
                For t=0 To Project\TOCColumns-1
                  WriteStringN(file,"  <col width="+Chr(34)+Str(100/Project\TOCColumns)+"%"+Chr(34)+">")
                Next
                WriteStringN(file,"</colgroup>")
                Col=0
                WriteStringN(file,"<tr>")
                ForEach *Include\Enumerations()
                  If *Include\Enumerations()\Chapter=*Include\Chapters()\Name
                    WriteStringN(file,"<td><a href="+Chr(34)+"#"+LCase(*Include\Enumerations()\Name)+Chr(34)+">"+*Include\Enumerations()\Name+"</a></td>")
                    Col+1
                    If Col>Project\TOCColumns-1
                      Col=0
                      WriteStringN(file,"</tr><tr>")
                    EndIf
                  EndIf
                Next
                WriteStringN(file,"</tr></table>")
              EndIf
            EndIf
            
            If ListSize(*Include\Structures())>0 And Project\Option_WriteStructures
              Found=0
              ForEach *Include\Structures()
                If *Include\Structures()\Chapter=*Include\Chapters()\Name
                  Found=1
                  Break
                EndIf
              Next
              If Found
                WriteStringN(file,"<h4>Structures</h4>")
                WriteStringN(file,"<table border=1 width="+Chr(34)+"100%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" cellspacing=0 rules="+Chr(34)+"none"+Chr(34)+">")
                WriteStringN(file,"<colgroup>")
                For t=0 To Project\TOCColumns-1
                  WriteStringN(file,"  <col width="+Chr(34)+Str(100/Project\TOCColumns)+"%"+Chr(34)+">")
                Next
                WriteStringN(file,"</colgroup>")
                Col=0
                WriteStringN(file,"<tr>")
                ForEach *Include\Structures()
                  If *Include\Structures()\Chapter=*Include\Chapters()\Name
                    WriteStringN(file,"<td><a href="+Chr(34)+"#"+LCase(*Include\Structures()\Name)+Chr(34)+">"+*Include\Structures()\Name+"</a></td>")
                    Col+1
                    If Col>Project\TOCColumns-1
                      Col=0
                      WriteStringN(file,"</tr><tr>")
                    EndIf
                  EndIf
                Next
                WriteStringN(file,"</tr></table>")
              EndIf
            EndIf
            
            If ListSize(*Include\Procedures())>0 And Project\Option_WriteProcedures
              Found=0
              ForEach *Include\Procedures()
                If *Include\Procedures()\Chapter=*Include\Chapters()\Name
                  Found+1
                EndIf
              Next
              If Found
                WriteStringN(file,"<h4>Procedures</h4>")
                WriteStringN(file,"<table border=1 width="+Chr(34)+"100%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" cellspacing=0 rules="+Chr(34)+"none"+Chr(34)+">")
                WriteStringN(file,"<colgroup>")
                For t=0 To Project\TOCColumns-1
                  WriteStringN(file,"  <col width="+Chr(34)+Str(100/Project\TOCColumns)+"%"+Chr(34)+">")
                Next
                WriteStringN(file,"</colgroup>")
                Col=0
                WriteStringN(file,"<tr>")
                ForEach *Include\Procedures()
                  If *Include\Procedures()\Chapter=*Include\Chapters()\Name
                    Decl.s=StringField(*Include\Procedures()\Declaration,1,"(")+"()"
                    WriteStringN(file,"<td><a href="+Chr(34)+"#"+LCase(Decl)+Chr(34)+">"+Decl+"</a></td>")
                    Col+1
                    If Col>Project\TOCColumns-1
                      Col=0
                      WriteStringN(file,"</tr><tr>")
                    EndIf
                  EndIf
                Next
                WriteStringN(file,"</tr></table>")
              EndIf
            EndIf
            
          Next      

          WriteStringN(file,"<br><hr size="+Chr(34)+"3"+Chr(34)+" align="+Chr(34)+"center"+Chr(34)+">")
        EndIf
        
        ; Write chapters
        ForEach *Include\Chapters()
          WriteStringN(file,"<center>")
          If *Include\Chapters()\Name<>""
            WriteStringN(file,"<h2>"+*Include\Chapters()\Name+"</h2>")
          EndIf
          If *Include\Chapters()\Description<>""
            WriteStringN(file,*Include\Chapters()\Description+"<br>")
          EndIf
          WriteStringN(file,"</center>")
        
          ;Write Enumerations
          If Project\Option_WriteEnumerations
            Found=0
            ForEach *Include\Enumerations()
              If *Include\Enumerations()\Chapter=*Include\Chapters()\Name
                Found=1
                Break
              EndIf
            Next
            If Found
              WriteStringN(file,"<h3>Enumerations</h3>")
              ForEach *Include\Enumerations()
                If *Include\Enumerations()\Chapter=*Include\Chapters()\Name
                  WriteStringN(file,"<h4><a name="+Chr(34)+LCase(*Include\Enumerations()\Name)+Chr(34)+">"+*Include\Enumerations()\Name+"</a></h4>")
                  
                  If *Include\Enumerations()\Description<>""
                    WriteStringN(file,ReplaceString(*Include\Enumerations()\Description,Chr(10),"<br>")+"<br>")
                  EndIf
        
                  WriteStringN(file,"<table border=1 width="+Chr(34)+"50%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" bgcolor="+Chr(34)+ColorToHex(Project\HTML_Enumeration_BGColor)+Chr(34)+" bordercolor="+Chr(34)+ColorToHex(Project\HTML_Enumeration_TextColor)+Chr(34)+" cellspacing=0>")
        
                  WriteStringN(file,"<colgroup>")
                  WriteStringN(file,"  <col width="+Chr(34)+"30%"+Chr(34)+">")
                  WriteStringN(file,"  <col width="+Chr(34)+"70%"+Chr(34)+">")
                  WriteStringN(file,"</colgroup>")
        
                  WriteStringN(file,"<tr>")
                    WriteStringN(file,"<th>Constant</th>")
                    WriteStringN(file,"<th>Description</th>")
                  WriteStringN(file,"</tr>")
                  ForEach *Include\Enumerations()\Constant()
                    WriteStringN(file,"<tr>")
                    WriteStringN(file,"<td>")
                    WriteStringN(file,*Include\Enumerations()\Constant()\Name)
                    WriteStringN(file,"</td><td>")
                    WriteStringN(file,"  "+ReplaceString(*Include\Enumerations()\Constant()\Description,Chr(10),"<br>"))
                    WriteStringN(file,"</td>")
                    WriteStringN(file,"</tr>")
                  Next
                  WriteStringN(file,"</table>")
                EndIf
              Next
            EndIf
          EndIf
          
          ;Write Structures
          If Project\Option_WriteStructures
            Found=0
            ForEach *Include\Structures()
              If *Include\Structures()\Chapter=*Include\Chapters()\Name
                Found=1
                Break
              EndIf
            Next
            If Found
              WriteStringN(file,"<h3>Structures</h3>")
              ForEach *Include\Structures()
                If *Include\Structures()\Chapter=*Include\Chapters()\Name
                  WriteStringN(file,"<h4><a name="+Chr(34)+LCase(*Include\Structures()\Name)+Chr(34)+">"+*Include\Structures()\Name+"</a></h4>")
                  
                  If *Include\Structures()\Description<>""
                    WriteStringN(file,ReplaceString(*Include\Structures()\Description,Chr(10),"<br>")+"<br>")
                  EndIf
                EndIf
              Next
            EndIf
          EndIf
          
          ;Write Procedures
          If Project\Option_WriteProcedures
            Found=0
            ForEach *Include\Procedures()
              If *Include\Procedures()\Chapter=*Include\Chapters()\Name
                Found=1
                Break
              EndIf
            Next
            If Found
              WriteStringN(file,"<h3>Procedures</h3>")
              ForEach *Include\Procedures()
                If *Include\Procedures()\Chapter=*Include\Chapters()\Name
                  WriteStringN(file,"<h4><a name="+Chr(34)+LCase(StringField(*Include\Procedures()\Declaration,1,"(")+"()")+Chr(34)+">"+StringField(*Include\Procedures()\Declaration,1,"(")+"()"+"</a></h4>")
                  
                  Text.s=""
                  If *Include\Procedures()\Author<>""
                    Text+"Author: "+*Include\Procedures()\Author+"     "
                  EndIf
                  If *Include\Procedures()\Version<>""
                    Text+"Version: "+*Include\Procedures()\Version
                  EndIf
                  If Text<>""
                    WriteStringN(file,Text+"<br><br>")
                  EndIf
                  
                  If ListSize(*Include\Procedures()\Image())>0
                    ForEach *Include\Procedures()\Image()
                      WriteStringN(file,"<img src="+Chr(34)+*Include\Procedures()\Image()+Chr(34)+">")
                    Next
                    WriteStringN(file,"<br>")
                  EndIf
                  
                  If *Include\Procedures()\Description<>""
                    WriteStringN(file,ReplaceString(*Include\Procedures()\Description,Chr(10),"<br>")+"<br>")
                  EndIf
                  
                  WriteStringN(file,"<b><u>Syntax:</u></b><br>")
                  WriteStringN(file,*Include\Procedures()\Declaration+"<br><br>")

                  If ListSize(*Include\Procedures()\Param())>0
                    WriteStringN(file,"<b><u>Parameters:</u></b><br>")
                    WriteStringN(file,"<table border=1 width="+Chr(34)+"50%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" bgcolor="+Chr(34)+ColorToHex(Project\HTML_Parameter_BGColor)+Chr(34)+" bordercolor="+Chr(34)+ColorToHex(Project\HTML_Parameter_TextColor)+Chr(34)+" cellspacing=0>")
          
                    WriteStringN(file,"<colgroup>")
                    WriteStringN(file,"  <col width="+Chr(34)+"30%"+Chr(34)+">")
                    WriteStringN(file,"  <col width="+Chr(34)+"70%"+Chr(34)+">")
                    WriteStringN(file,"</colgroup>")
          
                    WriteStringN(file,"<tr>")
                      WriteStringN(file,"<th>Parameter</th>")
                      WriteStringN(file,"<th>Description</th>")
                    WriteStringN(file,"</tr>")
                    ForEach *Include\Procedures()\Param()
                      WriteStringN(file,"<tr>")
                      WriteStringN(file,"<td>")
                      WriteStringN(file,*Include\Procedures()\Param()\Name)
                      WriteStringN(file,"</td><td>")
                      WriteStringN(file,ReplaceString(*Include\Procedures()\Param()\Description,Chr(10),"<br>"))
                      WriteStringN(file,"</td>")
                      WriteStringN(file,"</tr>")
                    Next
                    WriteStringN(file,"</table><br>")
                  EndIf
                  
                  If ListSize(*Include\Procedures()\Attribute())>0
                    WriteStringN(file,"<b><u>Supported Attributes:</u></b><br>")
                    WriteStringN(file,"<table border=1 width="+Chr(34)+"50%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" bgcolor="+Chr(34)+ColorToHex(Project\HTML_Attribute_BGColor)+Chr(34)+" bordercolor="+Chr(34)+ColorToHex(Project\HTML_Attribute_TextColor)+Chr(34)+" cellspacing=0>")
          
                    WriteStringN(file,"<colgroup>")
                    WriteStringN(file,"  <col width="+Chr(34)+"30%"+Chr(34)+">")
                    WriteStringN(file,"  <col width="+Chr(34)+"20%"+Chr(34)+">")
                    WriteStringN(file,"  <col width="+Chr(34)+"50%"+Chr(34)+">")
                    WriteStringN(file,"</colgroup>")
          
                    WriteStringN(file,"<tr>")
                      WriteStringN(file,"<th>Attribute</th>")
                      WriteStringN(file,"<th>Read/Write</th>")
                      WriteStringN(file,"<th>Description</th>")
                    WriteStringN(file,"</tr>")
                    ForEach *Include\Procedures()\Attribute()
                      WriteStringN(file,"<tr>")
                      WriteStringN(file,"<td>")
                      WriteStringN(file,*Include\Procedures()\Attribute()\Name)
                      WriteStringN(file,"</td><td>")
                      WriteStringN(file,*Include\Procedures()\Attribute()\Rights)
                      WriteStringN(file,"</td><td>")
                      WriteStringN(file,ReplaceString(*Include\Procedures()\Attribute()\Description,Chr(10),"<br>"))
                      WriteStringN(file,"</td>")
                      WriteStringN(file,"</tr>")
                    Next
                    WriteStringN(file,"</table><br>")
                  EndIf
                  
                  If ListSize(*Include\Procedures()\Event())>0
                    WriteStringN(file,"<b><u>Supported Events:</u></b><br>")
                    Text.s=""
                    ForEach *Include\Procedures()\Event()
                      Text+*Include\Procedures()\Event()\Name+", "
                    Next
                    WriteStringN(file,Left(Text,Len(Text)-2)+"<br><br>")
;                     WriteStringN(file,"<table border=1 width="+Chr(34)+"50%"+Chr(34)+" frame="+Chr(34)+"void"+Chr(34)+" bgcolor="+Chr(34)+ColorToHex(Project\HTML_Event_BGColor)+Chr(34)+" bordercolor="+Chr(34)+ColorToHex(Project\HTML_Event_TextColor)+Chr(34)+" cellspacing=0>")
;           
;                     WriteStringN(file,"<colgroup>")
;                     WriteStringN(file,"  <col width="+Chr(34)+"30%"+Chr(34)+">")
;                     WriteStringN(file,"  <col width="+Chr(34)+"70%"+Chr(34)+">")
;                     WriteStringN(file,"</colgroup>")
;           
;                     WriteStringN(file,"<tr>")
;                       WriteStringN(file,"<th>Event</th>")
;                       WriteStringN(file,"<th>Description</th>")
;                     WriteStringN(file,"</tr>")
;                     ForEach *Include\Procedures()\Event()
;                       WriteStringN(file,"<tr>")
;                       WriteStringN(file,"<td>")
;                       WriteStringN(file,*Include\Procedures()\Event()\Name)
;                       WriteStringN(file,"</td><td>")
;                       WriteStringN(file,ReplaceString(*Include\Procedures()\Event()\Description,Chr(10),"<br>"))
;                       WriteStringN(file,"</td>")
;                       WriteStringN(file,"</tr>")
;                     Next
;                     WriteStringN(file,"</table><br>")
                  EndIf
                  
                  If *Include\Procedures()\ReturnValue<>""
                    WriteStringN(file,"<b><u>Returns:</u></b><br>")
                    WriteStringN(file,*Include\Procedures()\ReturnValue+"<br><br>")
                  EndIf
                  
                  If *Include\Procedures()\Code<>""
                    WriteStringN(file,"<b><u>Example:</u></b><br>")
                    WriteStringN(file,"<pre>"+ReplaceString(*Include\Procedures()\Code,Chr(10),"<br>")+"</pre><br>")
                  EndIf
                  
                  If ListSize(*Include\Procedures()\Link())>0
                    WriteStringN(file,"<b><u>See also:</u></b><br>")
                    Text.s=""
                    ForEach *Include\Procedures()\Link()
                      If Left(*Include\Procedures()\Link()\Type,4)="proc" Or Left(*Include\Procedures()\Link()\Type,4)="enum"
                        Text+"<a href="+Chr(34)+"#"+LCase(*Include\Procedures()\Link()\Link)+Chr(34)+">"+*Include\Procedures()\Link()\Link+"</a>, "
                      ElseIf Left(*Include\Procedures()\Link()\Type,3)="ext"
                        Text+"<a href="+Chr(34)+LCase(*Include\Procedures()\Link()\Link)+Chr(34)+">"+*Include\Procedures()\Link()\Link+"</a>, "
                      EndIf
                    Next
                    WriteStringN(file,Left(Text,Len(Text)-2)+"<br><br>")
                  EndIf
        
                  WriteStringN(file,"<hr width="+Chr(34)+"20%"+Chr(34)+" size="+Chr(34)+"1"+Chr(34)+" align="+Chr(34)+"left"+Chr(34)+">")
                EndIf
              Next
  
              WriteStringN(file,"<br><hr size="+Chr(34)+"3"+Chr(34)+" align="+Chr(34)+"center"+Chr(34)+">")
            EndIf
          EndIf
          
        Next
  
      WriteStringN(file,"</body>")
      WriteStringN(file,"</html>")
      
      CloseFile(file)
    EndIf
  EndProcedure
  
  Procedure SaveProject(Filename.s)
    file=CreateFile(#PB_Any,Filename)
    If file
      WriteStringN(file,Str(Project\Option_WriteTOC))
      WriteStringN(file,Str(Project\Option_WriteAttributes))
      WriteStringN(file,Str(Project\Option_WriteEnumerations))
      WriteStringN(file,Str(Project\Option_WriteProcedures))
      WriteStringN(file,Str(Project\Option_WriteStructures))
      WriteStringN(file,Str(Project\Option_OneFile))
      WriteStringN(file,Str(Project\TOCColumns))

      WriteStringN(file,Str(Project\HTML_BGColor))
      WriteStringN(file,Str(Project\HTML_TextColor))
      WriteStringN(file,Str(Project\HTML_LinkColor))
      WriteStringN(file,Str(Project\HTML_LinkColor_Visited))
      WriteStringN(file,Str(Project\HTML_LinkColor_Active))
      WriteStringN(file,Str(Project\HTML_Parameter_BGColor))
      WriteStringN(file,Str(Project\HTML_Parameter_TextColor))
      WriteStringN(file,Str(Project\HTML_Enumeration_BGColor))
      WriteStringN(file,Str(Project\HTML_Enumeration_TextColor))
      WriteStringN(file,Str(Project\HTML_Attribute_BGColor))
      WriteStringN(file,Str(Project\HTML_Attribute_TextColor))

      ; Backup lines
      For t=0 To 9
        WriteStringN(file,"")
      Next

      ForEach Project\Include()
        WriteStringN(file,Project\Include()\File)
      Next

      CloseFile(file)
    EndIf
  EndProcedure
  
  Procedure LoadProject(Filename.s)
    file=ReadFile(#PB_Any,Filename)
    If file
      Project\Option_WriteTOC=Val(ReadString(file))
      Project\Option_WriteAttributes=Val(ReadString(file))
      Project\Option_WriteEnumerations=Val(ReadString(file))
      Project\Option_WriteProcedures=Val(ReadString(file))
      Project\Option_WriteStructures=Val(ReadString(file))
      Project\Option_OneFile=Val(ReadString(file))
      Project\TOCColumns=Val(ReadString(file))
      
      Project\HTML_BGColor=Val(ReadString(file))
      Project\HTML_TextColor=Val(ReadString(file))
      Project\HTML_LinkColor=Val(ReadString(file))
      Project\HTML_LinkColor_Visited=Val(ReadString(file))
      Project\HTML_LinkColor_Active=Val(ReadString(file))
      Project\HTML_Parameter_BGColor=Val(ReadString(file))
      Project\HTML_Parameter_TextColor=Val(ReadString(file))
      Project\HTML_Enumeration_BGColor=Val(ReadString(file))
      Project\HTML_Enumeration_TextColor=Val(ReadString(file))
      Project\HTML_Attribute_BGColor=Val(ReadString(file))
      Project\HTML_Attribute_TextColor=Val(ReadString(file))

      ; Backup lines
      For t=0 To 9
        ReadString(file)
      Next

      While Not Eof(file)
        AddElement(Project\Include())
        Project\Include()\File=ReadString(file)
      Wend
      
      CloseFile(file)
    EndIf
  EndProcedure

  Procedure NewProject()
    Project\Option_WriteTOC=1
    Project\Option_WriteAttributes=1
    Project\Option_WriteEnumerations=1
    Project\Option_WriteProcedures=1
    Project\Option_WriteStructures=1
    Project\Option_OneFile=1
    Project\TOCColumns=5
    
    Project\HTML_BGColor=RGB(220,220,255)
    Project\HTML_TextColor=RGB(0,0,0)
    
    Project\HTML_LinkColor=RGB(0,0,200)
    Project\HTML_LinkColor_Visited=RGB(0,0,255)
    Project\HTML_LinkColor_Active=RGB(20,20,255)
    
    Project\HTML_Parameter_BGColor=RGB(120,120,255)
    Project\HTML_Parameter_TextColor=RGB(0,0,0)
    
    Project\HTML_Enumeration_BGColor=RGB(120,255,120)
    Project\HTML_Enumeration_TextColor=RGB(0,0,0)
    
    Project\HTML_Attribute_BGColor=RGB(200,200,120)
    Project\HTML_Attribute_TextColor=RGB(0,0,0)
    
    ClearList(Project\Include())
    UI_ClearGadgetItems(#Table_Includes)
  EndProcedure
    
  ;- Main Loop
  
  NewProject()
  UpdateWindow()
  
  Repeat
    Repeat 
      UI_Update()
      
      If UI\Event=#UI_Event_Gadget

        If UI\EventType=#UI_EventType_LeftCLick

          If UI\EventGadget=#Button_AddInclude
            AddInclude()
            UpdateWindow()

          ElseIf UI\EventGadget=#Button_DelInclude
            DeleteInclude()
            UpdateWindow()

          ElseIf UI\EventGadget=#Button_Up
            SelectElement(Project\Include(),UI_GetGadgetState(#Table_Includes))
            First=@Project\Include()
            SelectElement(Project\Include(),UI_GetGadgetState(#Table_Includes)-1)
            Second=@Project\Include()
            SwapElements(Project\Include(),First,Second)
            
            NewState=UI_GetGadgetState(#Table_Includes)-1  
            
            UI_ClearGadgetItems(#Table_Includes)
            ForEach Project\Include()
              UI_AddGadgetItem(#Table_Includes,-1,GetFilePart(Project\Include()\File))
              UI_SetGadgetItemData(#Table_Includes,-1,@Project\Include())
            Next
            UI_SetGadgetState(#Table_Includes,NewState)
            UpdateWindow()

          ElseIf UI\EventGadget=#Button_Down
            SelectElement(Project\Include(),UI_GetGadgetState(#Table_Includes))
            First=@Project\Include()
            SelectElement(Project\Include(),UI_GetGadgetState(#Table_Includes)+1)
            Second=@Project\Include()
            SwapElements(Project\Include(),First,Second)
            
            NewState=UI_GetGadgetState(#Table_Includes)+1  
            
            UI_ClearGadgetItems(#Table_Includes)
            ForEach Project\Include()
              UI_AddGadgetItem(#Table_Includes,-1,GetFilePart(Project\Include()\File))
              UI_SetGadgetItemData(#Table_Includes,-1,@Project\Include())
            Next
            UI_SetGadgetState(#Table_Includes,NewState)
            UpdateWindow()

          ElseIf UI\EventGadget=#Button_BGColor
            Color=ColorRequester(Project\HTML_BGColor)
            If Color>-1
              Project\HTML_BGColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_TextColor
            Color=ColorRequester(Project\HTML_TextColor)
            If Color>-1
              Project\HTML_TextColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_LinkColor
            Color=ColorRequester(Project\HTML_LinkColor)
            If Color>-1
              Project\HTML_LinkColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_LinkVisitedColor
            Color=ColorRequester(Project\HTML_LinkColor_Visited)
            If Color>-1
              Project\HTML_LinkColor_Visited=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_LinkActiveColor
           Color=ColorRequester(Project\HTML_LinkColor_Active)
            If Color>-1
              Project\HTML_LinkColor_Active=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_ParameterBGColor
            Color=ColorRequester(Project\HTML_Parameter_BGColor)
            If Color>-1
              Project\HTML_Parameter_BGColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_ParameterTextColor
            Color=ColorRequester(Project\HTML_Parameter_TextColor)
            If Color>-1
              Project\HTML_Parameter_TextColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_EnumerationBGColor
            Color=ColorRequester(Project\HTML_Enumeration_BGColor)
            If Color>-1
              Project\HTML_Enumeration_BGColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_EnumerationTextColor
            Color=ColorRequester(Project\HTML_Enumeration_TextColor)
            If Color>-1
              Project\HTML_Enumeration_TextColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_Attribute_BGColor
            Color=ColorRequester(Project\HTML_Attribute_BGColor)
            If Color>-1
              Project\HTML_Attribute_BGColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_Attribute_TextColor
            Color=ColorRequester(Project\HTML_Attribute_TextColor)
            If Color>-1
              Project\HTML_Attribute_TextColor=Color
            EndIf
            UpdateWindow()
            
          ElseIf UI\EventGadget=#Button_New
            If MessageRequester("New Project","Are you sure you want to start a new project?",#PB_MessageRequester_YesNo)=#PB_MessageRequester_Yes
              NewProject()
              UpdateWindow()
            EndIf

          ElseIf UI\EventGadget=#Button_Load
            Filename.s=OpenFileRequester("Load Project","","DocMaker Projects|*.dmp",0)
            If Filename
              NewProject()
              LoadProject(Filename)
              ForEach Project\Include()
                UI_AddGadgetItem(#Table_Includes,-1,GetFilePart(Project\Include()\File))
                UI_SetGadgetItemData(#Table_Includes,-1,@Project\Include())
              Next
            EndIf

          ElseIf UI\EventGadget=#Button_Save
            Filename.s=SaveFileRequester("Save Project","","DocMaker Projects|*.dmp",0)
            If Filename
              If LCase(Right(Filename,4))<>".dmp"
                Filename+".dmp"
              EndIf
              SaveProject(Filename)
            EndIf

          ElseIf UI\EventGadget=#Button_Export
            Destination.s=SaveFileRequester("Output file","","HTML file|*.html",0)
            If Destination<>""
              If LCase(Right(Destination,5))<>".html"
                Destination+".html"
              EndIf
              ClearIncludes()
              For t=0 To UI_CountGadgetItems(#Table_Includes)-1
                ForEach Project\Include()
                  If @Project\Include()=UI_GetGadgetItemData(#Table_Includes,t)
                    ScanInclude(@Project\Include(),Project\Include()\File)
                  EndIf
                Next
              Next
              ExportHTML(Destination)
            EndIf

          ElseIf UI\EventGadget=#Table_Includes
            UpdateWindow()

          ElseIf UI\EventGadget=#Checkbox_WriteTOC
            Project\Option_WriteTOC=UI_GetGadgetState(#Checkbox_WriteTOC)

          ElseIf UI\EventGadget=#Checkbox_WriteProcedures
            Project\Option_WriteProcedures=UI_GetGadgetState(#Checkbox_WriteProcedures)

          ElseIf UI\EventGadget=#Checkbox_WriteEnumerations
            Project\Option_WriteEnumerations=UI_GetGadgetState(#Checkbox_WriteEnumerations)

          EndIf

        EndIf

      ElseIf UI\Event=#PB_Event_CloseWindow
        UI\Quit=1

      EndIf

    Until UI\Event=0
        
    UI_Draw()
    Delay(10)
  Until UI\Quit
    
CompilerEndIf
; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 34
; Folding = --
; EnableXP